import numpy as np
import pandas as pd

from scipy.stats import gaussian_kde
from scipy.interpolate import interp1d

def create_pdf(data, bins=200):
    x_vals = np.linspace(data.min(), data.max(), num=bins)
    kde = gaussian_kde(data)
    return x_vals, kde(x_vals)

def map_density_range(new_range, old_range, old_pdf):
    pdf_interp = interp1d(old_range, old_pdf, kind='linear', fill_value=0.0, bounds_error=False)
    return pdf_interp(new_range)

def kl_divergence(p, q):
    is_finite = (p != 0) & (q != 0)
    p, q = p[is_finite], q[is_finite]
    return np.sum(p * (np.log2(p) - np.log2(q)))

def best_kl_fit(signature, pdfs, bins=200):
    xv, pdf = create_pdf(signature, bins)
    res = []
    pdf_keys = list(pdfs.keys())
    for k in pdf_keys:
        ipdf = map_density_range(xv, pdfs[k]["xvalues"], pdfs[k]["pdf"])
        k = (kl_divergence(pdf, ipdf)+kl_divergence(ipdf,pdf))/2
        res.append(k)
    min_pos = np.argmin(res)
    return res[min_pos], pdf_keys[min_pos]



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="blitzgsea",
    version="1.3.32",
    author="Alexander Lachmann",
    author_email="alexander.lachmann@mssm.edu",
    description="Package for fast and accurate calculation of Gene Set Enrichment Analysis (GSEA) similar to prerank using gamma distribution approximation.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/maayanlab/blitzgsea",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    package_data={
        "blitzgsea": ["data/*"]
    },
    include_package_data=True,
    install_requires=[
        'pandas>=1.1.5',
        'numpy',
        'scikit-learn',
        'tqdm',
        'statsmodels',
        'mpmath'
    ],
    python_requires='>=3.6',
)
